/*:
 * @plugindesc CustomWindowMessage
 * @author YourName
 *
 * @command show
 * @text ウィンドウを表示
 * @desc ウィンドウを表示します。
 *
 * @arg x
 * @type number
 * @text X座標
 * @desc ウィンドウのX座標
 * @default 0
 *
 * @arg y
 * @type number
 * @text Y座標
 * @desc ウィンドウのY座標
 * @default 0
 *
 * @arg width
 * @type number
 * @text ウィンドウの幅
 * @desc ウィンドウの幅
 * @default 300
 *
 * @arg height
 * @type number
 * @text ウィンドウの高さ
 * @desc ウィンドウの高さ
 * @default 72
 *
 * @arg text
 * @type multiline_string
 * @text 表示するメッセージ
 * @desc 表示するメッセージ
 *
 * @arg fontSize
 * @type number
 * @text フォントサイズ
 * @desc テキストのフォントサイズ
 * @default 28
 *
 * @arg textColor
 * @type string
 * @text テキストカラー
 * @desc テキストのカラー（HTMLカラーコード形式）
 * @default #ffffff
 *
 * @arg duration
 * @type number
 * @text 表示時間 (フレーム数)
 * @desc ウィンドウが表示される時間 (フレーム数)
 * @default 120
 *
 * @arg fadeInDuration
 * @type number
 * @text フェードインの時間
 * @desc ウィンドウがフェードインする時間 (フレーム数)
 * @default 10
 *
 * @arg fadeOutDuration
 * @type number
 * @text フェードアウトの時間
 * @desc ウィンドウがフェードアウトする時間 (フレーム数)
 * @default 10
 *
 * @help
 * このプラグインは、指定した位置にウィンドウを表示し、指定した時間経過後に消える機能を提供します。
 * 複数のウィンドウを同時に表示でき、後から表示されるウィンドウは、前のウィンドウの下に連なります。
 * ウィンドウが消えると、その下にあるウィンドウが上にスライドし、消えたウィンドウの位置で止まります。
 */
class CustomWindowMessage extends Window_Base {
    constructor(x, y, width, height, text, fontSize, textColor, duration, fadeInDuration, fadeOutDuration) {
      super(new Rectangle(x, y, width, height));
      this._text = text;
      this._fontSize = fontSize;
      this._textColor = textColor;
      this._duration = duration;
      this._fadeInDuration = fadeInDuration;
      this._fadeOutDuration = fadeOutDuration;
      this.opacity = 0;
      this.refresh();
    }
  
    refresh() {
      this.contents.clear();
      this.contents.fontSize = this._fontSize;
      this.changeTextColor(this._textColor);
      this.drawText(this._text, 0, 0, this.width, 'center');
    }
  
    update() {
      super.update();
      if (this._duration > 0) {
        this._duration--;
        if (this._duration <= this._fadeOutDuration) {
          this.opacity -= 255 / this._fadeOutDuration;
        } else if (this.opacity < 255 && this._duration > this._fadeInDuration) {
          this.opacity += 255 / this._fadeInDuration;
        }
      } else {
        this.parent.removeChild(this);
      }
    }
  }
  
  PluginManager.registerCommand('CustomWindowMessage', 'show', function (args) {
    const x = Number(args.x);
    const y = Number(args.y);
    const width = Number(args.width);
    const height = Number(args.height);
    const text = String(args.text);
    const fontSize = Number(args.fontSize);
    const textColor = String(args.textColor);
    const duration = Number(args.duration);
    const fadeInDuration = Number(args.fadeInDuration);
    const fadeOutDuration = Number(args.fadeOutDuration);
  
    const customWindow = new CustomWindowMessage(
      x, y, width, height, text, fontSize, textColor, duration, fadeInDuration, fadeOutDuration
    );
  
    SceneManager._scene.children[0].addChild(customWindow);
  });
  