/*:
 * @target MZ
 * @plugindesc ランダムパターン選択プラグイン
 * @author YourName
 * 
 * @command ramdomMoney
 * @text ランダムな投げ銭額を出力
 * @desc テストだよ。
 *
 * @param Variable1
 * @text 100～400円
 * @desc 100～400円の確率を決定する変数のID
 * @type number
 * @default 1
 *
 * @param Variable2
 * @text 500～900円
 * @desc 500～900円の確率を決定する変数のID
 * @type number
 * @default 2
 *
 * @param Variable3
 * @text 1000～1900円
 * @desc 1000～1900円の確率を決定する変数のID
 * @type number
 * @default 3
 *
 * @param Variable4
 * @text 2000～4000円
 * @desc 2000～4000円の確率を決定する変数のID
 * @type number
 * @default 4
 *
 * @param Variable5
 * @text 5000～9000円
 * @desc 5000～9000円の確率を決定する変数のID
 * @type number
 * @default 5
 *
 * @param Variable6
 * @text 10000～50000円
 * @desc 10000～50000円の確率を決定する変数のID
 * @type number
 * @default 6
 * 
 * @param 50000_hosei_id
 * @type variable
 * @text 10000~50000の確率補正用変数のID
 * @desc 10000~50000の確率補正用変数のID
 * @default 0
 * 
 * @param 9000_hosei_id
 * @type variable
 * @text 5000~9000の確率補正用変数のID
 * @desc 5000~9000の確率補正用変数のID
 * @default 0
 * 
 * @arg pattern_id
 * @type variable
 * @text 投げ銭パターンの収納場所
 * @desc 投げ銭パターンの収納場所
 * @default 34
 * 
 * @arg Money_id
 * @type variable
 * @text 投げ銭額の収納場所
 * @desc 投げ銭額の収納場所
 * @default 11
 * 
 *
 *
 * @help
 * このプラグインは、6つの変数に基づいてパターンをランダムに決定し、結果を別の変数に格納する機能を提供します。
 * プラグインコマンドを使用してイベントで実行することができます。
 * 
 * プラグインコマンド:
 * RandomPatternPlugin decidePattern resultVariable x
 * - x は結果を格納する変数のIDです。
 */


(() => {
    "use strict";

    const pluginName = document.currentScript.src.match(/^.*\/(.*).js$/)[1];
    const parameters = PluginManager.parameters(pluginName);
    const variableIds = [
        Number(parameters['Variable1']) || 1,
        Number(parameters['Variable2']) || 2,
        Number(parameters['Variable3']) || 3,
        Number(parameters['Variable4']) || 4,
        Number(parameters['Variable5']) || 5,
        Number(parameters['Variable6']) || 6
    ];
    const hosei_50000 = parameters['50000_hosei_id'];
    const hosei_9000 = parameters['9000_hosei_id'];

    function decidePattern() {
        let totalWeight = 0;

        let copy_variableIds = variableIds;
        copy_variableIds[5] = $gameVariables.value(hosei_50000);
        copy_variableIds[4] = $gameVariables.value(hosei_9000);
        
        // 各変数の値を合計して、totalWeightに格納します。
        for (let i = 0; i < variableIds.length; i++) {
            totalWeight += parseFloat((copy_variableIds[i]));
        }

        //showMessage.call( this, totalWeight);

        // totalWeightを基に乱数を生成します。
        const randomValue = Math.random() * totalWeight;
        let currentWeight = 0;

        // 乱数の値によって、選択されたパターンを決定します。
        for (let i = 0; i < copy_variableIds.length; i++) {
            currentWeight += parseFloat((copy_variableIds[i]));
            if (randomValue < currentWeight) {
                return i ;
            }
        }

        return 0;
    }

    function decideMoney(pattern) {
        let Money = 0;
        let random = 1;

        if(pattern == 0){
            random = 1+Math.floor(Math.random() * (4));
            Money = random * 100;
        }
        else if(pattern == 1){
            random = Math.floor(Math.random() * (5));
            Money = random * 100 + 500;
        }
        else if(pattern == 2){
            random = Math.floor(Math.random() * (10));
            Money = random * 100 + 1000;
        }
        else if(pattern == 3){
            random = Math.floor(Math.random() * (3));
            Money = random * 1000 + 2000;
        }
        else if(pattern == 4){
            random = Math.floor(Math.random() * (5));
            Money = random * 1000 + 5000;
        }
        else if(pattern == 5){
            random = Math.floor(Math.random() * (5));
            Money = random * 10000 + 10000;
        }

        return Money;
    }

  
    // プラグインコマンドを定義します。
    PluginManager.registerCommand(pluginName, 'ramdomMoney', function(args) {
        // 結果を格納する変数のIDを取得します。
        const resultVariable = Number(args.pattern_id) || 0;
        // 結果の投げ銭額を格納する変数のIDを取得します。
        const resultMoney_ID = Number(args.Money_id) || 1;

        // decidePattern関数を呼び出し、結果を得ます。
        const pattern = decidePattern();

        //showMessage.call( this, pattern);
        // 結果を指定された変数に格納します。
        $gameVariables.setValue(resultVariable, pattern);

        const Money = decideMoney(pattern);

        $gameVariables.setValue(resultMoney_ID, Money);
        
    });

    function showMessage( messageText ) {
        $gameMessage.add( messageText ); // メッセージを設定して
        this.setWaitMode( 'message' ); // 入力待ち状態にします
    }

})();